from os import  environ, makedirs
from os.path import isfile, join, isdir
from typing import List
from ipaddress import ip_address
from toml import load as toml_load
from json import load, dump
from json.decoder import JSONDecodeError
from pprint import pprint


MULTI_FUNCTIONS = []
EXTRA_ANNOTATORS = ['risotto.rougail']
ROUGAIL_NAMESPACE = 'general'
ROUGAIL_NAMESPACE_DESCRIPTION = 'Général'
HERE = environ['PWD']
IP_DIR = join(HERE, 'ip')


# custom filters from dataset
custom_filters = {}


config_file = environ.get('CONFIG_FILE', 'risotto.conf')
if isfile(config_file):
    with open(config_file, 'r') as fh:
        RISOTTO_CONFIG = toml_load(fh)
else:
    RISOTTO_CONFIG = {}


def _(s):
    return s


def multi_function(function):
    global MULTI_FUNCTIONS
    name = function.__name__
    if name not in MULTI_FUNCTIONS:
        MULTI_FUNCTIONS.append(name)
    return function


def value_pprint(dico, config):
    pprint_dict = {}
    for path, value in dico.items():
        if config.option(path).type() == 'password' and value:
            value = 'X' * len(value)
        pprint_dict[path] = value
    pprint(pprint_dict)


def load_zones(zones, hosts):
    if not isdir(IP_DIR):
        makedirs(IP_DIR)
    json_file = join(IP_DIR, 'zones.json')
    if isfile(json_file):
        try:
            with open(json_file, 'r') as fh:
                ori_zones_ip = load(fh)
        except JSONDecodeError:
            ori_zones_ip = {}
    else:
        ori_zones_ip = {}
    new_zones_ip = {}
    # cache, machine should not change IP
    for host_name, dhosts in hosts.items():
        for server_name, server in dhosts['servers'].items():
            server_zones = server['zones_name']
            for idx, zone_name in enumerate(server_zones):
                zone = zones[zone_name]
                zone.setdefault('hosts', {})
                if zone_name not in new_zones_ip:
                    new_zones_ip[zone_name] = {}
                if zone_name in ori_zones_ip and server_name in ori_zones_ip[zone_name]:
                    server_index = ori_zones_ip[zone_name][server_name]
                    if server_index >= zone['length']:
                        server_index = None
                    elif server_index in new_zones_ip[zone_name].values():
                        server_index = None
                else:
                    server_index = None
                new_zones_ip[zone_name][server_name] = server_index
    for zone_name, servers in new_zones_ip.items():
        for server_name, server_idx in servers.items():
            if server_idx is not None:
                continue
            for new_idx in range(zones[zone_name]['length']):
                if new_idx not in new_zones_ip[zone_name].values():
                    new_zones_ip[zone_name][server_name] = new_idx
                    break
            else:
                raise Exception(f'cannot find free IP in zone "{zone_name}" for "{server_name}"')
    for zone_name, servers in new_zones_ip.items():
        start_ip = ip_address(zones[zone_name]['start_ip'])
        for server_name, server_index in servers.items():
            zones[zone_name]['hosts'][server_name] = str(start_ip + server_index)
    with open(json_file, 'w') as fh:
        dump(new_zones_ip, fh)
